'''
    An example script showing how to get data from reconstructed root file and decode the cellID column
    * Commented code uses dd4hep decoder which will take a significant amount of time for initialization
    ** ReadoutDecoder class solely depends on python xml parser and is thus much faster, but it may not work
       if some of the features are not supported by the parser (but supported by dd4hep parser)
    Chao Peng (ANL)
'''
import os
import ROOT
import pandas as pd
import numpy as np
import argparse
from matplotlib import pyplot as plt
import matplotlib.ticker as ticker
from utils import ReadoutDecoder, flatten_collection


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('rec_file', help='Path to reconstruction output file.')
    parser.add_argument('-o', dest='outdir', default='.', help='Output directory.')
    parser.add_argument('-c', '--compact', dest='compact', required=True,
            help='Top-level xml file of the detector description')
    parser.add_argument('--collection', dest='coll', required=True,
            help='Hits collection name in the reconstruction file')
    parser.add_argument('--readout', dest='readout', required=True,
            help='Readout name for the hits collection')
    args = parser.parse_args()

    # decoder
    decoder = ReadoutDecoder(args.compact, args.readout)

    # get hits
    rdf_rec = ROOT.RDataFrame('events', args.rec_file)
    df = flatten_collection(rdf_rec, args.coll)
    df.rename(columns={c: c.replace(args.coll + '.', '') for c in df.columns}, inplace=True)

    # initialize dd4hep detector
    # import DDG4
    # kernel = DDG4.Kernel()
    # description = kernel.detectorDescription()
    # kernel.loadGeometry("file:{}".format(args.compact))
    # dd4hep_decoder = description.readout(args.readout).idSpec().decoder()
    # lindex = dd4hep_decoder.index('x')
    # get_layer_id = np.vectorize(lambda cid: dd4hep_decoder.get(cid, lindex))
    # df.loc[:, 'layerID'] = get_layer_id(df['cellID'].astype(int).values)
    # always terminate dd4hep kernel
    # kernel.terminate()

    # faster way to get layerids
    df.loc[:, 'layerID'] = decoder.get(df['cellID'].values, 'layer')
    df.loc[:, 'zID'] = decoder.get(df['cellID'].values, 'z')
    print(df[['cellID', 'layerID', 'zID', 'position.x', 'position.y', 'position.z', 'energy']])

