from Gaudi.Configuration import *

from Configurables import ApplicationMgr, EICDataSvc, PodioOutput, GeoSvc
from GaudiKernel import SystemOfUnits as units

detector_path = str(os.environ.get("DETECTOR_PATH", "."))
detector_name = str(os.environ.get("DETECTOR_CONFIG", "epic"))
detector_config = str(os.environ.get("DETECTOR_CONFIG", detector_name))
detector_version = str(os.environ.get("DETECTOR_VERSION", "main"))

# todo add checks
input_sim_file = str(os.environ["JUGGLER_SIM_FILE"])
output_rec_file = str(os.environ["JUGGLER_REC_FILE"])
n_events = str(os.environ["JUGGLER_N_EVENTS"])

## note: old version of material map is called material-maps.XXX, new version is materials-map.XXX
##       these names are somewhat inconsistent, and should probably all be renamed to 'material-map.XXX'
##       FIXME
geo_service = GeoSvc(
    "GeoSvc",
    detectors=["{}/{}.xml".format(detector_path, detector_name)],
    materials="calibrations/materials-map.cbor",
    OutputLevel=WARNING,
)
podioevent = EICDataSvc("EventDataSvc", inputs=[input_sim_file], OutputLevel=WARNING)

from Configurables import PodioInput

from Configurables import Jug__Digi__SimTrackerHitsCollector as SimTrackerHitsCollector
from Configurables import Jug__Digi__SiliconTrackerDigi as TrackerDigi

from Configurables import (
    Jug__Reco__TrackerHitReconstruction as TrackerHitReconstruction,
)
from Configurables import Jug__Reco__TrackingHitsCollector2 as TrackingHitsCollector

from Configurables import Jug__Reco__TrackerSourceLinker as TrackerSourceLinker
from Configurables import Jug__Reco__TrackParamTruthInit as TrackParamTruthInit
from Configurables import Jug__Reco__TrackParamClusterInit as TrackParamClusterInit
from Configurables import (
    Jug__Reco__TrackParamVertexClusterInit as TrackParamVertexClusterInit,
)

from Configurables import Jug__Reco__CKFTracking as CKFTracking
from Configurables import Jug__Reco__ParticlesFromTrackFit as ParticlesFromTrackFit

from Configurables import Jug__Reco__SimpleClustering as SimpleClustering


algorithms = []

tracker_endcap_collections = [
    "TrackerEndcapHits",
]
tracker_barrel_collections = [
    "SiBarrelHits",
]
vertex_barrel_collections = [
    "VertexBarrelHits",
]
mpgd_barrel_collections = [
    "MPGDBarrelHits",
]

input_collections = (
    ["MCParticles"]
    + tracker_endcap_collections
    + tracker_barrel_collections
    + vertex_barrel_collections
    + mpgd_barrel_collections
)

podioinput = PodioInput("PodioReader", collections=input_collections)
algorithms.append(podioinput)

trk_b_coll = SimTrackerHitsCollector(
    "trk_b_coll",
    inputSimTrackerHits=tracker_barrel_collections,
    outputSimTrackerHits="TrackerBarrelAllHits",
)
algorithms.append(trk_b_coll)
trk_b_digi = TrackerDigi(
    "trk_b_digi",
    inputHitCollection=trk_b_coll.outputSimTrackerHits,
    outputHitCollection="TrackerBarrelRawHits",
    timeResolution=8,
)
algorithms.append(trk_b_digi)

trk_ec_coll = SimTrackerHitsCollector(
    "trk_ec_coll",
    inputSimTrackerHits=tracker_endcap_collections,
    outputSimTrackerHits="TrackerEndcapAllHits",
)
algorithms.append(trk_ec_coll)
trk_ec_digi = TrackerDigi(
    "trk_ec_digi",
    inputHitCollection=trk_ec_coll.outputSimTrackerHits,
    outputHitCollection="TrackerEndcapRawHits",
    timeResolution=8,
)
algorithms.append(trk_ec_digi)

vtx_b_coll = SimTrackerHitsCollector(
    "vtx_b_coll",
    inputSimTrackerHits=vertex_barrel_collections,
    outputSimTrackerHits="VertexBarrelAllHits",
)
algorithms.append(vtx_b_coll)
vtx_b_digi = TrackerDigi(
    "vtx_b_digi",
    inputHitCollection=vtx_b_coll.outputSimTrackerHits,
    outputHitCollection="VertexBarrelRawHits",
    timeResolution=8,
)
algorithms.append(vtx_b_digi)

mpgd_b_coll = SimTrackerHitsCollector(
    "mpgd_b_coll",
    inputSimTrackerHits=mpgd_barrel_collections,
    outputSimTrackerHits="MPGDTrackerBarrelAllHits",
)
algorithms.append(mpgd_b_coll)
mpgd_b_digi = TrackerDigi(
    "mpgd_b_digi",
    inputHitCollection=mpgd_b_coll.outputSimTrackerHits,
    outputHitCollection="MPGDTrackerBarrelRawHits",
    timeResolution=8,
)
algorithms.append(mpgd_b_digi)

# Tracker and vertex reconstruction
trk_b_reco = TrackerHitReconstruction(
    "trk_b_reco",
    inputHitCollection=trk_b_digi.outputHitCollection,
    outputHitCollection="TrackerBarrelRecHits",
)
algorithms.append(trk_b_reco)

trk_ec_reco = TrackerHitReconstruction(
    "trk_ec_reco",
    inputHitCollection=trk_ec_digi.outputHitCollection,
    outputHitCollection="TrackerEndcapRecHits",
)
algorithms.append(trk_ec_reco)

vtx_b_reco = TrackerHitReconstruction(
    "vtx_b_reco",
    inputHitCollection=vtx_b_digi.outputHitCollection,
    outputHitCollection="VertexBarrelRecHits",
)
algorithms.append(vtx_b_reco)

mpgd_b_reco = TrackerHitReconstruction(
    "mpgd_b_reco",
    inputHitCollection=mpgd_b_digi.outputHitCollection,
    outputHitCollection="MPGDTrackerBarrelRecHits",
)
algorithms.append(mpgd_b_reco)

input_tracking_hits = [
    str(trk_b_reco.outputHitCollection),
    str(trk_ec_reco.outputHitCollection),
    str(vtx_b_reco.outputHitCollection),
    str(mpgd_b_reco.outputHitCollection),
]

trk_hit_col = TrackingHitsCollector(
    "trk_hit_col",
    inputTrackingHits=input_tracking_hits,
    trackingHits="trackingHits",
)
algorithms.append(trk_hit_col)

# Hit Source linker
sourcelinker = TrackerSourceLinker(
    "sourcelinker",
    inputHitCollection=trk_hit_col.trackingHits,
    outputSourceLinks="TrackSourceLinks",
    outputMeasurements="TrackMeasurements",
)
algorithms.append(sourcelinker)

## Track param init
truth_trk_init = TrackParamTruthInit(
    "truth_trk_init",
    inputMCParticles="MCParticles",
    outputInitialTrackParameters="InitTrackParams",
)
algorithms.append(truth_trk_init)

# Tracking algorithms
trk_find_alg = CKFTracking(
    "trk_find_alg",
    inputSourceLinks=sourcelinker.outputSourceLinks,
    inputMeasurements=sourcelinker.outputMeasurements,
    inputInitialTrackParameters="InitTrackParams",
    outputTrajectories="trajectories",
)
algorithms.append(trk_find_alg)

parts_from_fit = ParticlesFromTrackFit(
    "parts_from_fit",
    inputTrajectories="trajectories",
    outputParticles="ReconstructedParticles",
    outputTrackParameters="outputTrackParameters",
)
algorithms.append(parts_from_fit)

# types = []
## this printout is useful to check that the type information is passed to python correctly
# print("---------------------------------------\n")
# print("---\n# List of input and output types by class")
# for configurable in sorted([ PodioInput, EICDataSvc, PodioOutput,
#                             TrackerHitReconstruction,ExampleCaloDigi,
#                             UFSDTrackerDigi, TrackerSourceLinker,
#                             PodioOutput],
#                           key=lambda c: c.getType()):
#    print("\"{}\":".format(configurable.getType()))
#    props = configurable.getDefaultProperties()
#    for propname, prop in sorted(props.items()):
#        print(" prop name: {}".format(propname))
#        if isinstance(prop, DataHandleBase):
#            types.append(prop.type())
#            print("  {}: \"{}\"".format(propname, prop.type()))
# print("---")

out = PodioOutput("out", filename=output_rec_file)
out.outputCommands = [
    "keep *",
    "drop BarrelTrackSourceLinks",
    "drop InitTrackParams",
    "drop trajectories",
    "drop outputSourceLinks",
    "drop outputInitialTrackParameters",
    "keep MCParticles",
]
algorithms.append(out)

ApplicationMgr(
    TopAlg=algorithms,
    EvtSel="NONE",
    EvtMax=n_events,
    ExtSvc=[podioevent, geo_service],
    OutputLevel=WARNING,
)
